from mongogogo.schema import *
import pytest
import datetime

def test_datetime():
    s = DateTime()
    v = s.serialize(datetime.datetime.utcnow())
    assert v <= datetime.datetime.utcnow()
    assert isinstance(v, datetime.datetime)

def test_date():
    s = Date()
    v = s.serialize(datetime.date(2012, 3, 17))
    assert v.day == 17
    assert v.month == 3
    assert v.year == 2012
    assert isinstance(v, datetime.date)

def test_date_from_datetime():
    s = Date()
    v = s.serialize(datetime.datetime(2012, 3, 17, 18, 2))
    assert v.day == 17
    assert v.month == 3
    assert v.year == 2012
    assert isinstance(v, datetime.date)

def test_integer():
    s = Integer()
    v = s.serialize(2)
    assert v == 2

def test_integer_from_string():
    s = Integer()
    v = s.serialize("2")
    assert v == 2

def test_integer_min_ok():
    s = Integer(min=17)
    v = s.serialize(17)
    assert v == 17
    
def test_integer_min_not_ok():
    s = Integer(min=17)
    pytest.raises(Invalid, lambda : s.serialize(16))

def test_integer_max_ok():
    s = Integer(max=17)
    v = s.serialize(16)
    assert v == 16
    
def test_integer_max_not_ok():
    s = Integer(max=17)
    pytest.raises(Invalid, lambda : s.serialize(19))

def test_invalid_integer():
    s = Integer()
    pytest.raises(Invalid, lambda : s.serialize("a"))

def test_float():
    s = Float()
    v = s.serialize(2.3)
    assert v == 2.3

def test_float_from_string():
    s = Float()
    v = s.serialize("2.3")
    assert v == 2.3

def test_invalid_float():
    s = Float()
    pytest.raises(Invalid, lambda : s.serialize("a"))

def test_float_min_ok():
    s = Float(min=17.6)
    v = s.serialize(17.61)
    assert v == 17.61
    
def test_float_min_not_ok():
    s = Float(min=17.6)
    pytest.raises(Invalid, lambda : s.serialize(17.59))

def test_float_max_ok():
    s = Float(max=17.6)
    v = s.serialize(17.59)
    assert v == 17.59
    
def test_float_max_not_ok():
    s = Float(max=17.6)
    pytest.raises(Invalid, lambda : s.serialize(18.59))





from utils import null, Invalid, marker, AttributeMapper
import datetime

__all__ = [
    "SchemaNode",
    "Schema",
    "String",
    "Integer",
    "Float",
    "Boolean",
    "Date",
    "DateTime",
    "Dict",
    "List",
]

###
### nodes
###

class SchemaNode(object):
    """base class for all schema nodes which have to derive from it.

    It handles marking this node as schema node and doing instance counting
    in case the schema needs a defined sequence of nodes.
    """

    _schemanode = True # marker for meta class that this is part of the schema
    _counter = 0 # this is a global counter which is incremented on instantiation of a type.

    def __new__(cls, *args, **kwargs):
        # increment the counter so we know which the original sequence of type nodes is. 
        SchemaNode._counter += 1
        instance = super(SchemaNode, cls).__new__(cls, *args, **kwargs)
        instance._counter = SchemaNode._counter

        # now collect the nodes in this instance
        instance._nodes = []
        for name in dir(instance):
            if not name.startswith('_'):
                field = getattr(instance, name)

                # filter out only the type elements. We have marker in the base class for that
                if hasattr(field, '_schemanode'):
                    field.name = name
                    instance._nodes.append((name, field))
        return instance

    def __init__(self, on_serialize = [], on_deserialize = [], default = marker, required = False, name = None): 
        """initialize the ``SchemaNode`` with generic parameters like queues, default and required flag

        :param on_serialize: a list of filters to be run before the actual serialization
        :param on_deserialize: a list of filters to be run before the actual deserialization
        :param default: If given this is the default for both serialization and deserialization. If you want
            separate values for serialization and deserialization use the ``Default`` filter in the
            respective filter queue. Also note that setting a default and required to True makes no sense
            as default is set before required is tested
        :param required: Define whether the field is required in both serialization and deserialization. The
            same like for default applies regarding separate queues. 
        :param name: the name of the field. In case it is included in a schema this will be set automatically
        """
        self.on_serialize = on_serialize
        self.on_deserialize = on_deserialize
        self.default = default
        self.required = required
        self.name = name


    def serialize(self, value = null, data = null, **kw):
        """serialize data to a data structure for MongoDB. If this is a combined node then 
        serialization on all sub nodes is called as well with it's respective value (or null in case
        it does not exist in the source data.

        The sequence is as follows:
        - first all filters in ``on_serialize`` are applied to the data. 
            Each filter is called with the ``value``, ``data`` and any passed additional keywords.
        - then the default check is done. If no value is available and ``default`` is set then this is used
        - lastly the required check is done.

        .. note:: Is this sequence right?

        :param value: The individual value for this node to serialize
        :param data: The whole record e.g. in case you need access to a different field for e.g. a 
            password test in a filter. Note that the this record is always the source record and not
            the serialized version as this can not be complete.  
        :param kw: Additional keywords which will be passed to the actual serialization code and filters
        :return: The serialized version of the data.
        
        """

        # this is for bootstrapping the master schema / mapping
        if data is null: 
            data = value

        for filter in self.on_serialize:
            value = filter(value, data, **kw)

        # check default
        if value is null and self.default is not marker:
            if callable(self.default):
                value = self.default()
            else:
                value = self.default

        # check required
        if value is null and self.required:
            raise Invalid(self, "required data missing")

        # now run the actual serialization
        return self.do_serialize(value, data, **kw)

    def deserialize(self, value = null, data = null, **kw):
        """deserialize MongoDB data to a python usable data structure. If this is a combined node then 
        deserialization on all sub nodes is called as well with it's respective value (or null in case
        it does not exist in the source data.)

        It will return the deserialized data of the complete source value.

        :param value: The individual value for this node to deserialize
        :param data: The whole record e.g. in case you need access to a different field for e.g. a 
            password test in a filter. Note that the this record is always the source record and not
            the deserialized version as this can not be complete.  
        :param kw: Additional keywords which will be passed to the actual deserialization code and filters
        :return: The deserialized version of the data.
        
        """

        # this is for bootstrapping the master schema / mapping
        if data is null: data = value

        for filter in self.on_deserialize:
            value = filter(value, data, **kw)

        # check default
        if value is null and self.default is not marker:
            value = self.default

        # check required
        if value is null and self.required:
            raise Invalid(self, "required data missing")

        # now run the actual deserialization
        return self.do_deserialize(value, data, **kw)

    def do_serialize(self, value, data, **kw):
        """the actual serialization code which you have to override in your own node implementations.
        The implementation has to return the single serialized value.
        """
        return value

    def do_deserialize(self, value, data, **kw):
        """the actual deserialization code which you have to override in your own node implementations.
        The implementation has to return the single deserialized value.
        """
        return value


class Schema(SchemaNode):

    def do_serialize(self, value, data = null, **kw):
        """serialize mapping data"""

        output = {} # of course we have a mapping as output
        for name, field in self._nodes:
            # TODO: here exceptions!
            sub_value = value.get(name, null)
            output[name] = field.serialize(sub_value, data = data, **kw)
        return output

    def deserialize(self, value, data = null, **kw):
        """deserialize from MongoDB to Python"""

        output = {} # of course we have a mapping as output
        for name, field in self._nodes:
            # TODO: here exceptions!
            sub_value = value.get(name, null)
            output[name] = field.deserialize(sub_value, data = data, **kw)
        return output

class String(SchemaNode):
    """a string type. """

    def __init__(self, encoding = None, *args, **kw):
        """initialize the String Type"""
        super(String, self).__init__(*args, **kw)
        self.encoding = encoding

    def do_serialize(self, value, data, **kw):
        """serialize data"""
        if value is None:
            if self.required:
                raise Invalid(self, "required data missing")
            else:
                # return something mongo compatible so not the null object
                return None
        try:
            if isinstance(value, unicode):
                if self.encoding:
                    result = value.encode(self.encoding)
                else:
                    result = value
            else:
                encoding = self.encoding
                if encoding:
                    result = unicode(value, encoding).encode(encoding)
                else:
                    result = unicode(value)
            return result
        except Exception, e:
            raise Invalid(self, "Value '%s' cannot be serialized: %s" %(value, e))


class Date(SchemaNode):
    """a date type which converts DateTime objects to Date objects"""

    def do_serialize(self, value, data, **kw):
        if isinstance(value, datetime.datetime):
            value = value.date()
        elif value is None and not self.required:
            return None
        elif value is None and self.required:
            raise Invalid(self, "required data missing")
        elif not isinstance(value, datetime.date):
            raise Invalid(self, "Value '%s' is not an instance of datetime.datetime.Date" %(value))
        # we need to return a datetime object as mongo does not understand something else
        return datetime.datetime.combine(value, datetime.time())

    def do_deserialize(self, value, data, **kw):
        """convert datetime back to date"""
        if value is None:
            return value
        return value.date()

class DateTime(SchemaNode):
    """a datetime type. """

    def do_serialize(self, value, data, **kw):
        if isinstance(value, datetime.date):
            value = datetime.datetime.combine(value, datetime.time())
        elif value is None and not self.required:
            return None
        elif value is None and self.required:
            raise Invalid(self, "required data missing")
        elif not isinstance(value, datetime.date):
            raise Invalid(self, "Value '%s' is not an instance of datetime.datetime" %(value))
        return value


class Dict(SchemaNode):
    """a dict type. """

    def __init__(self, dotted = False, *args, **kw):
        """initialize the Dict Type.

        :param dotted: if set to ``True`` then this dictionary will also allow dotted access using AttributeMapper
        
        """
        super(Dict, self).__init__(*args, **kw)
        self.dotted = dotted

    def do_deserialize(self, value, data, **kw):
        """deserialize either into a normal dictionary or into an AttributeMapper allowing dotted notation
        """
        if self.dotted:
            return AttributeMapper(value)
        else:
            return dict(value)


class Integer(SchemaNode):
    """an integer type. """

    def __init__(self, min = None, max = None, *args, **kw):
        """initialize the String Type"""
        super(Integer, self).__init__(*args, **kw)
        self.min = min
        self.max = max

    def do_serialize(self, value, data, **kw):
        """serialize data"""
        if value is null and not self.required:
            return None
        try:
            v = int(value)
        except Exception, e: 
            raise Invalid(self, "Value '%s' cannot be serialized: %s" %(value, e))
        if self.min is not None and v < self.min:
            raise Invalid(self, "Value '%s' is too low, minimum value is %s" %(value, self.min))
        if self.max is not None and v > self.max:
            raise Invalid(self, "Value '%s' is too big, maximum value is %s" %(value, self.max))
        return v

class Float(Integer):
    """a float type. """

    def do_serialize(self, value, data, **kw):
        """serialize data"""
        if value is null and not self.required:
            return None
        try:
            v = float(value)
        except Exception, e: 
            raise Invalid(self, "Value '%s' cannot be serialized: %s" %(value, e))

        if self.min is not None and v < self.min:
            raise Invalid(self, "Value '%s' is too low, minimum value is %s" %(value, self.min))
        if self.max is not None and v > self.max:
            raise Invalid(self, "Value '%s' is too big, maximum value is %s" %(value, self.max))
        return v

class Boolean(SchemaNode):
    """an integer type. """

    def do_serialize(self, value, data, **kw):
        """serialize data"""
        if value is null and not self.required:
            return False
        try:
            v = bool(value)
        except Exception, e: 
            raise Invalid(self, "Value '%s' cannot be serialized: %s" %(value, e))
        return v

class List(SchemaNode):
    """a node which consists of a list of sub nodes"""

    def __init__(self, subtype, *args, **kw):
        """initialize the list with a list of subtypes which are allowed to be in the document"""
        super(List, self).__init__(*args, **kw)
        self.subtype = subtype

    def do_serialize(self, value, data, **kw):
        """serialize the list"""
        if value is null and not self.required:
            value = []
        elif value is null and self.required:
            raise Invalid(self, "required data missing")
        result = []
        for item in value:
            result.append(self.subtype.serialize(item, data, **kw))
        return result
    



import pytest
import datetime

def test_add(db, persons):
    p = persons.data_class(firstname="Foo", lastname="Bar")
    persons.save(p)
    assert p._id is not None

def test_get(db, persons):
    p = persons.data_class(firstname="Foo", lastname="Bar")
    persons.save(p)
    p2 = persons[p._id]
    assert p2.lastname == "Bar"

def test_defaults(db, persons):
    p = persons.data_class()
    assert p.lastname == "foobar"

def test_callable_defaults(db, persons):
    p = persons.data_class()
    assert isinstance(p.creation,datetime.datetime)

def test_schemaless(db, schemaless_persons):
    p = schemaless_persons.data_class(firstname="Foo", lastname="Bar", extra=1)
    schemaless_persons.save(p)
    p2 = schemaless_persons[p._id]
    assert p2.extra == 1

def test_schemaenforce(db, persons):
    p = persons.data_class(firstname="Foo", lastname="Bar", extra=1)
    persons.save(p)
    p2 = persons[p._id]
    pytest.raises(AttributeError, lambda : p2.extra)

def test_callable_defaults(db, persons):
    p = persons.data_class()
    assert p.creation < datetime.datetime.utcnow()

def test_integerfield(db, persons):
    p = persons.data_class(firstname="Foo", lastname="Bar", age="24")
    persons.save(p)
    p2 = persons[p._id]
    assert p2.age == 24

def test_dictfield_not_dotted(db, persons):
    p = persons.data_class(firstname="Foo", lastname="Bar", e={'foo' : 'bar'})
    persons.save(p)
    p2 = persons[p._id]
    assert p2.e['foo'] == "bar"
    pytest.raises(AttributeError, lambda: p2.e.foo)


def test_dictfield_dotted(db, persons):
    p = persons.data_class(firstname="Foo", lastname="Bar", d={'foo' : 'bar'})
    persons.save(p)
    p2 = persons[p._id]
    assert p2.d.foo == "bar"

def test_find(db, persons): 
    # insert some records into the database
    for i in range(1,10):
        p = persons.data_class(firstname="Foo%s" %i, lastname="Bar%s" %i)
        persons.save(p)
    res = persons.find({'firstname':"Foo1"})
    assert res.count() == 1
    assert isinstance(res[0], persons.data_class)

def test_find_and_sort(db, persons): 
    # insert some records into the database
    for i in range(1,10):
        p = persons.data_class(firstname="Foo%s" %i, lastname="Bar%s" %i, age=17)
        persons.save(p)
    res = persons.find({'age':17}).sort("firstname", 1)
    assert res[0].firstname == "Foo1"
    res = persons.find({'age':17}).sort("firstname", -1)
    assert res[0].firstname == "Foo9"

def test_find_one(db, persons): 
    # insert some records into the database
    for i in range(1,10):
        p = persons.data_class(firstname="Foo%s" %i, lastname="Bar%s" %i)
        persons.save(p)
    person = persons.find_one({'firstname':"Foo1"})
    assert person.firstname == "Foo1"

def test_with_id(db, persons): 
    p = persons.data_class(firstname="Foo", lastname="Bar", _id=u"cs")
    persons.save(p)
    p2 = persons[u"cs"]
    assert p2._id == u"cs"

def test_with_id_and_find(db, persons): 
    p = persons.data_class(firstname="Foo", lastname="Bar", _id=u"cs")
    persons.save(p)
    p2 = persons.find_one({'_id' : u'cs'})
    assert p2._id == u"cs"

def test_deserialize_on_get(db, persons): 
    """check if the schema is properly deserialized on find(). We use a schema for it
    which increments the ``inc`` field by one on each retrieve."""
    p = persons.data_class(firstname="Foo", lastname="Bar", _id=u"cs")
    persons.save(p)
    p2 = persons.get(u'cs')
    assert p2.incr == 2
    persons.save(p2)
    p2 = persons.get(u'cs')
    assert p2.incr == 3

def test_deserialize_on_find_one(db, persons): 
    """check if the schema is properly deserialized on find(). We use a schema for it
    which increments the ``inc`` field by one on each retrieve."""
    p = persons.data_class(firstname="Foo", lastname="Bar", _id=u"cs")
    persons.save(p)
    p2 = persons.find_one({'_id' : u'cs'})
    assert p2.incr == 2
    persons.save(p2)
    p2 = persons.find_one({'_id' : u'cs'})
    assert p2.incr == 3

def test_initialize_with_doc(db, persons):
    # this initially gave an error because we serialized first
    # before storing the doc
    p = persons.data_class(firstname="Foo")
    persons.save(p)
    p2 = persons[p._id]
    assert p2.firstname == "Foo"




import uuid
import types
import copy
from cursor import Cursor

class AttributeMapper(dict):
    """a dictionary like object which also is accessible via getattr/setattr"""

    __slots__ = []

    def __init__(self, default={}, *args, **kwargs):
        super(AttributeMapper, self).__init__(*args, **kwargs)
        self.update(default)
        self.update(kwargs)

    def __getattr__(self, k):
        """retrieve some data from the dict"""
        if self.has_key(k):
            return self[k]
        raise AttributeError(k)

    def __setattr__(self, k,v):
        """store an attribute in the map"""
        self[k] = v

    def _clone(self):
        """return a clone of this object"""
        d = copy.deepcopy(self) 
        return AttributeMapper(d)

    def update(self, d):
        """update the dictionary but make sure that existing included AttributeMappers are only updated aswell"""
        for a,v in d.items():
            if a not in self:
                self[a] = v
            elif isinstance(self[a], AttributeMapper) and type(v) == types.DictType:
                self[a].update(v)
            elif type(self[a]) == types.DictType and type(v) == types.DictType:
                self[a].update(v)
            else:
                self[a] = v

class DatabaseError(Exception):
    """master class for all mongogogo exceptions"""

class CollectionMissing(DatabaseError):
    """raised if a collection is missing or closed""" 

class InvalidData(DatabaseError):
    """exception raised if the data is invalid on serialize"""

    def __init__(self, errors):
        """initialize the exception.

        :param errors: A dictionary containing the field names associated with messages

        """
        self.errors = errors

    def __str__(self):
        """return a printable representation of the errors"""
        fs = ["%s: %s" %(a,v) for a,v in self.errors.items()]
        return """<Invalid Data: %s>""" %", ".join(fs)

class ObjectNotFound(DatabaseError):
    """exception raised if an object was not found"""

    def __init__(self, _id):
        """initialize the exception"""
        self._id = _id


class Record(dict):
    
    schema = None
    _protected = ['schema', 'collection', '_protected', '_schemaless', 'default_values']
    schemaless = False # set to true to allow arbitrary data. If set to False, then additional data will be filtered out
    default_values = {} # default values for a newly created record. Will only be used if from_db is None 

    def __init__(self, doc={}, from_db = None, collection = None, *args, **kwargs):
        """initialize a record with data

        :param doc: The initial document coming from python. This will be merged with keyword
            arguments but won't get serialized or deserialized with the schema as it's already python.
            If ``doc`` and ``from_db`` are both given then only from_db will be used. 
        :param from_db: a record coming from the database. This will be deserialized. If it's None then
            we assume that it's a new object. Note that default values from the schema will not be set.
            This will only happen on the serialization step on save.
        :param collection: the collection instance this data object belongs to
        """

        self._id = None
        if self.schemaless:
            super(Record, self).__init__(from_db if from_db is not None else doc, *args, **kwargs)
        else:
            super(Record, self).__init__(*args, **kwargs)

        # only deserialize it if it's coming from the database
        if from_db is not None:
            self.update(self.schema.deserialize(from_db))
            self._id = from_db.get("_id", None)
        else:
            self._initialize_defaults()
            self.update(doc)
            self.update(kwargs)

        self._collection = collection

        if from_db is None:
            # lets initialize it
            self.after_initialize()
            self.after_create()
        else:
            self.after_load()

    def _initialize_defaults(self):
        """initialize the record with the default values"""
        def ini(value):
            if callable(value):
                value = value()
            if type(value) == types.DictType:
                for a,v in value.items():
                    value[a] = ini(v)
            if type(value) == types.ListType:
                n = []
                for v in value:
                    n.append(ini(v))
                value = n
            return value
        self.update(ini(self.default_values))

    def __getattr__(self, k):
        """retrieve some data from the dict"""
        if k in self._protected:
            return dict.__getattribute__(self, k)
        if self.has_key(k):
            return self[k]
        raise AttributeError(k)

    def __setattr__(self, k,v):
        """store an attribute in the map"""
        if k in self._protected:
            dict.__setattr__(self, k, v)
        else:
            self[k] = v

    def update(self, d):
        """update the dictionary but make sure that existing included AttributeMappers are only updated aswell"""
        for a,v in d.items():
            if a not in self:
                self[a] = v
            elif isinstance(self[a], Record) and type(v) == types.DictType:
                self[a].update(v)
            elif type(self[a]) == types.DictType and type(v) == types.DictType:
                self[a].update(v)
            else:
                self[a] = v
    
    def after_initialize(self):
        """initialize this object. This method is called when it's a new object meaning
        that it has no unique id"""
        pass

    def after_create(self):
        """hook, which is called after an object has been created and defaults have been initialized"""
        pass

    def after_load(self):
        """hook, which is called after an object has been loaded from the database"""
        pass

    def save(self):
        """save this record"""
        if self._collection is None:
            raise CollectionMissing()
        self._collection.save(self)

    put = save

    def remove(self):
        """remove this record"""
        if self._collection is None:
            raise CollectionMissing()
        self._collection.remove(self)
            

class Collection(object):
    """collection class for handling objects"""

    data_class = Record
    create_ids = False # if True then you can override gen_id to generate a new id, otherwise a UUID will be used. If False then we use mongo objectids 
    convert_objectids = True # if True then get() will convert string _ids to object ids

    def __init__(self, collection, md = {}, **kwargs):
        """initialize the collection

        :param collection: The pymongo collection object to use
        :param md: Additional Metadata to be stored in this collection (link to some config etc. maybe useful for validation)
        :param kwargs: Additional parameters which will be stored inside the metadata dict
        """
        self.collection = collection
        self.md = AttributeMapper(md)
        self.md.update(kwargs)

    def new_id(self):
        """create a new unique id"""
        return unicode(uuid.uuid4())

    def put(self, obj):
        """store an object"""

        # check if we need to create an id
        _id = None
        if "_id" not in obj and self.create_ids:
            _id = self.new_id()
        else:
            _id = obj._id

        # now serialize and validate the object
        obj = self.before_serialize(obj)
        if obj.schemaless:
            data = obj
            data.update(obj.schema.serialize(obj))
        else:
            data = obj.schema.serialize(obj)
        if _id is not None:
            data['_id'] = _id
        data = self.before_put(obj, data) # hook for handling additional validation etc.
        self.collection.save(data, True)
        obj._id = data['_id']
        obj._collection = self
        self.after_put(obj)
        return obj

    save = put

    def before_serialize(self, obj):
        """hook for changing the object before it's serialized"""
        return obj

    def before_put(self, obj, data):
        """hook for handling additional validation etc."""
        return data

    def after_put(self, obj):
        """hook for changing data after the object from the database has been instantiated"""
        pass

    def get(self, _id):
        """return an object by it's id"""
        data = self.collection.find_one({'_id' : _id})
        if data is None:
            raise ObjectNotFound(_id)
        #if self.data_class.schemaless:
            #data.update(self.data_class.schema.deserialize(data))
        #else:
            #data = self.data_class.schema.deserialize(data)
        data['_id'] = _id
        return self.data_class(from_db = data, collection=self)

    def remove(self, obj):
        """high level method to remove an object"""
        q = {'_id' : obj._id}
        return self._remove(q)

    def _remove(self, *args, **kwargs):
        """raw remove method for using a query to remove one or more objects"""
        return self.collection.remove(*args, **kwargs)

    def find(self, *args, **kwargs):
        return Cursor(self, wrap = self.data_class, *args, **kwargs)
        
    def find_one(self, spec_or_id=None, *args, **kwargs):

        if spec_or_id is not None and not isinstance(spec_or_id, dict):
            spec_or_id = {"_id": spec_or_id}

        for result in self.find(spec_or_id, *args, **kwargs).limit(-1):
            return result
        return None

    __getitem__ = get
        




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2009-2012, Nicolas Clairon, Christian Scholz
# All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the University of California, Berkeley nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# This was copied from mongokit as it looked useful. It was changed slightly though to adapt it

from pymongo.cursor import Cursor as PymongoCursor
from collections import deque

class Cursor(PymongoCursor):
    def __init__(self, collection, *args, **kwargs):
        self.__wrap = None
        self.__mongogogo_collection = collection
        if kwargs:
            self.__wrap = kwargs.pop('wrap', None)
        super(Cursor, self).__init__(collection.collection, *args, **kwargs)

    def next(self):
        if self._Cursor__empty:
            raise StopIteration
        db = self._Cursor__collection.database
        if len(self.__data) or self._refresh():
            if isinstance(self._Cursor__data, deque):
                item = self._Cursor__data.popleft()
            else:
                item = self._Cursor__data.pop(0)
            if self._Cursor__manipulate:
                son = db._fix_outgoing(item, self._Cursor__collection)
            else:
                son = item
            if self.__wrap is not None:
                return self.__wrap(from_db = son, collection=self.__mongogogo_collection)
            else:
                return son
        else:
            raise StopIteration

    def __getitem__(self, index):
        obj = super(Cursor, self).__getitem__(index)
        if (self.__wrap is not None) and isinstance(obj, dict):
            return self.__wrap(from_db = obj)
        return obj
